from numbers import Number


def upRange(start, stop, step):
  while start <= stop:
    yield start
    start += abs(step)

def downRange(start, stop, step):
  while start >= stop:
    yield start
    start -= abs(step)

"""Somma elemento a elemento due vettori.
Parametri:
   a, b: i due vettori (liste) da sommare
Valore restituito:
un vettore (una lista) in cui ogni elemento è la somma
dei corrispondenti elementi dei vettori (liste) a e b
"""
def SommaVettori(a, b):
  aplusb = []
  i_end = float(len(a))
  for i in (1 <= i_end) and upRange(1, i_end, 1) or downRange(1, i_end, 1):
    aplusb.append(a[int(i - 1)] + b[int(i - 1)])
  return aplusb

"""Implementazione software di un cancellatore adattivo di rumore
acustico con algoritmo di filtraggio adattivo LMS (least mean square).
Parametri:
d: segnale catturato dal microfono primario, che include il segnale
vocale desiderato corrotto da rumore (una lista di numeri reali)
    v1: segnale di rumore (una lista di numeri reali)
    M: lunghezza del filtro
    mu: passo dell'algoritmo
Valore restituito:
    il segnale ripulito (una lista di numeri reali)
"""
def CancellazioneRumore(d, v2, M, mu):
  L = len(d)
  # Segnale ripulito da rumore che vogliamo ottenere.
  # Inizialmente una lista di elementi tutti pari a zero.
  e = [0] * L
  # Il filtro, inizialmente una lista di elementi tutti pari a zero
  w = [0] * M
  # Buffer per immagazzinare i campioni del segnale
  # di rumore. Una lista (inizialmente di elementi
  # tutti pari a zero) di lunghezza pari al filtro.
  v2temp = [0] * M
  # Il ciclo è ripetuto per ogni campione del
  # segnale del microfono primario (la lista d)
  for i in (1 <= float(L)) and upRange(1, float(L), 1) or downRange(1, float(L), 1):
    v2temp = AggiornamentoBuffer(i, v2, v2temp)
    # Si calcola il valore del campione del segnale di stima
    # del rumore al microfono primario, ottenuto attraverso la
    # moltiplicazione del filtro w[n - 1] con il buffer v2[n].
    v1stima = Convoluzione(w, v2temp)
    # Avendo ora a disposizione il valore del campione per il segnale
    # stimato di rumore v1stima per questa iterazione, possiamo
    # sottrarlo al campione corrispondente del segnale microfonico d,
    # ottenendo il valore del campione attuale del segnale ripulito da rumore.
    # Si sottrae al segnale del microfono primario d la stima del rumore v1stima.
    e[int(i - 1)] = d[int(i - 1)] - v1stima
    w = AggiornamentoFiltro(w, mu, e[int(i - 1)], v2temp)
  return e

"""Moltiplicazione righe per colonne del vettore riga (lista a)
per il vettore colonna (lista b). Somma il prodotto del primo
elemento di a per il primo elemento di b, il secondo ...
Parametri:
    a, b: i due vettori (liste) da moltiplicare
Valore restituito:
    uno scalare, il prodotto righe per colonne
"""
def Convoluzione(a, b):
  somma = 0
  k_end = float(len(a))
  for k in (1 <= k_end) and upRange(1, k_end, 1) or downRange(1, k_end, 1):
    somma = (somma if isinstance(somma, Number) else 0) + a[int(k - 1)] * b[int(k - 1)]
  return somma

"""Alla iterazione n = 0 il buffer è inizializzato a 0,
quindi tutti gli elementi del buffer v2 [n] sono nulli.
Alla iterazione n = 1 il buffer si muove di una posizione
e il primo elemento del buffer viene posto uguale al
primo valore del vettore che contiene il segnale di rumore
v1. All’iterazione n = 2 il buffer avrà i primi 2 elementi
del segnale di rumore, e così via fino all’iterazione
M dove il buffer conterrà i primi M campioni del
segnale di rumore. E’ bene notare che ad ogni iterazione
il buffer conterrà tutti gli elementi dell’iterazione
precedente ma spostati di una posizione. In particolare,
l’ultimo elemento viene rimosso, gli altri elementi vengono
spostati di una posizione indietro e la prima posizione
verrà occupata dal campione n-esimo del segnale di rumore.
Parametri:
    n: iterazione
    v2: segnale di rumore
    buffer: buffer
Valore restituito:
    il buffer aggiornato (una lista di lunghezza M)
"""
def AggiornamentoBuffer(n, v2, buffer2):
  j_start = float(len(buffer2))
  for j in (j_start <= 2) and upRange(j_start, 2, 1) or downRange(j_start, 2, 1):
    buffer2[int(j - 1)] = buffer2[int((j - 1) - 1)]
  buffer2[0] = v2[int(n - 1)]
  return buffer2

"""L’ultima operazione da effettuare all’interno del
ciclo for è l’aggiornamento del vettore del filtro
di cancellazione w[n] = w[n - 1]+ mu x e[n] x v2[n].
Parametri:
w: la lista dei campioni del filtro all'iterazione precedente
    mu: parametro costante
en: campione del segnale ripulito in questa iterazione (ultimo calcolato)
    v2temp: lista dei campioni di stima del rumore
Valore restituito
    il filtro aggiornato (una lista)
"""
def AggiornamentoFiltro(w, mu, en, v2temp):
  k_end2 = float(len(v2temp))
  for k in (1 <= k_end2) and upRange(1, k_end2, 1) or downRange(1, k_end2, 1):
    temp = (mu * en) * v2temp[int(k - 1)]
    w[int(k - 1)] = w[int(k - 1)] + temp
  return w


# Segnale vocale originario emesso dal parlatore (per uno scenario di simulazione)
# s = [1, 4, -1, -2, 0]
# Segnale di rumore
# v2 = [2, 3, 0, -2, 2]
# Segnale di rumore
# v1 = [1, 2, 1, -1, 1]
# Segnale catturato dal microfono primario, che include
# il segnale vocale desiderato corrotto da rumore.
# In uno scenario simulato è la somma del segnale vocale
# originario emesso dal parlatore s e del segnale di rumore v1.
# d = SommaVettori(s, v1)
d = [3.13338, 5.22177, 7.56687, 5.68092, 2.29853, -0.34064, -1.00972, -0.02355, 2.98681, 4.95838, 5.09001, 5.063, 0.68501, -1.96272, -2.61264, -1.13798, 1.16999, 4.67448, 4.24233, 1.09097, -1.17941, -4.26254, -4.92537, -2.86625, 0.37796, 1.69815, 2.8301, 0.27125, -3.70043, -5.65165, -5.56093, -4.7354, -1.01233, 0.9415, 0.19966, -1.11587, -4.99699, -7.25203, -5.61329, -3.21866, -0.86882, 1.40928, 0.19474, -3.01195, -4.65998, -5.74286, -5.74646, -2.36927]
v2 =[3.13338, 2.35234, 3.56857, 2.97906, 2.53203, 2.68657, 2.97494, 2.50151, 2.52061, 1.78371, 1.13258, 2.72333, 1.38232, 1.34859, 1.30407, 1.00831, 0.24395, 1.23783, 0.3797, -0.85463, -0.0278, -0.71227, -1.13, -1.12799, -0.9953, -1.95363, -0.88508, -1.25375, -2.11021, -1.91081, -1.93862, -3.42886, -2.81409, -2.87564, -3.31743, -2.19949, -2.98984, -3.37161, -2.21341, -2.36165, -3.07453, -2.52119, -3.07632, -3.63943, -2.26325, -1.77575, -2.61536, -1.97347] 
print(CancellazioneRumore(d, v2, 15, 0.001))